@@ -1,6 +1,6 @@
 ;; examples/nyacc/lang/c99/tryit.scm
 
-;; Copyright (C) 2020-2024 Matthew Wette
+;; Copyright (C) 2020-2025 Matthew Wette
 ;;
 ;; Copying and distribution of this file, with or without modification,
 ;; are permitted in any medium without royalty provided the copyright
@@ -20,6 +20,7 @@
 (use-modules (sxml fold))
 (use-modules ((sxml xpath) #:hide (filter)))
 
+(use-modules (nyacc lang c99 ffi-help))
 (use-modules (nyacc lang c99 parser))
 (use-modules (nyacc lang c99 cxeval))
 (use-modules (nyacc lang c99 pprint))
@@ -28,6 +29,7 @@
 (use-modules (nyacc lang c99 cpp))
 (use-modules (nyacc lang c99 util))
 (use-modules (nyacc foreign arch-info))
+(use-modules (nyacc foreign cdata))
 (use-modules (nyacc lang sx-util))
 (use-modules (nyacc lang util))
 (use-modules (nyacc lex))
@@ -49,9 +51,9 @@
   (call-with-output-string (lambda (port) (pretty-print-c99 exp port))))
 
 
-(define *cpp-defs* (get-gcc-cpp-defs))
-(define *inc-dirs* (get-gcc-inc-dirs))
-(define *inc-help* c99-def-help)
+(define *cpp-defs* (make-parameter (get-gcc-cpp-defs)))
+(define *inc-dirs* (make-parameter (get-gcc-inc-dirs)))
+(define *inc-help* (make-parameter c99-def-help))
 
 (define *mode* (make-parameter 'code))
 (define *debug* (make-parameter #f))
@@ -60,9 +62,9 @@
 (define* (parse-file file #:key cpp-defs inc-dirs mode debug)
   (with-input-from-file file
     (lambda ()
-      (parse-c99 #:cpp-defs (or cpp-defs *cpp-defs*)
-                 #:inc-dirs (or inc-dirs *inc-dirs*)
-                 #:inc-help *inc-help*
+      (parse-c99 #:cpp-defs (or cpp-defs (*cpp-defs*))
+                 #:inc-dirs (or inc-dirs (*inc-dirs*))
+                 #:inc-help (*inc-help*)
                  #:mode (or mode (*mode*))
                  #:debug (or debug (*debug*))
                  #:show-incs #f
@@ -74,9 +76,9 @@
   (with-input-from-string str
     (lambda ()
       (parse-c99 tyns
-                 #:cpp-defs (or cpp-defs *cpp-defs*)
-                 #:inc-dirs (or inc-dirs *inc-dirs*)
-                 #:inc-help *inc-help*
+                 #:cpp-defs (or cpp-defs (*cpp-defs*))
+                 #:inc-dirs (or inc-dirs (*inc-dirs*))
+                 #:inc-help (*inc-help*)
                  #:mode (or mode (*mode*))
                  #:debug (or debug (*debug*))
                  #:show-incs #f
@@ -85,9 +87,11 @@
 (define (parse-string-list . str-l)
   (parse-string (apply string-append str-l)))
 
+#|
 (define (fold p s l)
   (let loop ((s s) (l l))
     (if (null? l) s (loop (p (car l) s) (cdr l)))))
+|#
 
 (define-syntax-rule (pass-if mesg expr)
   (begin
@@ -368,7 +372,6 @@
 
 (when #f
   ;; munge's pointer-declr? was erroneously saying arrays are pointers
-  (use-modules (nyacc lang c99 ffi-help-cd))
   (let* ((code "
 typedef struct foo foo_t;
 typedef struct foo { int x; int y; };
@@ -382,5 +385,135 @@ typedef struct bar { foo_t a; foo_t b[2]; } bar_t;")
     (pperr xdecl)
     #f))
 
+(when #f
+  (let* (
+         ;; ugh: "typedef unsigned uint32_t;"
+         ;;(code "extern int version(void);")
+         ;;(code "typedef long unsigned int size_t;")
+         ;;(code "long long x = (long long) 1;")
+         #|
+         (code "typedef int foo; int f(foo foo);")
+         |#
+         (code (string-append
+                "typedef unsigned bar;"
+                "float f2(int x) {"
+                " typedef float bar;"
+                " bar y = 1.0;"
+                " return x + y;"
+                " }"
+                ))
+         ;;(code "typedef unsigned uint32_t;")
+         ;;(code "typedef int foo; int f(foo foo);")
+         ;;(code "typedef enum foo { FOO } foo_t; foo_t x;")
+         (code "typedef struct S S; struct S { S* S; } s;")
+         (x (sferr "~s\n" code))
+         (tree (parse-string code #:debug #t))
+         ;;(udict (c99-trans-unit->udict tree))
+         ;;(udecl (sx-ref* tree 2))
+         ;;(xdecl (expand-typerefs udecl udict '((enum . "foo"))))
+         )
+    (pperr tree)
+    ;;(pp udict)
+    ;;(pp udecl)
+    ;;(pp xdecl)
+    ))
+
+(when #f                                ; bug 63502
+  (let* (
+         (code "
+#include <glib.h>
+typedef GHashTable MyHashTable;
+int foo(MyHashTable *mht);
+")
+         (tree (parse-string
+                code
+                #:inc-dirs (cons* "/usr/include/glib-2.0"
+                                  "/usr/lib/x86_64-linux-gnu/glib-2.0/include"
+                                  "/usr/include/sysprof-6"
+                                  (get-sys-inc-dirs)))))
+    (pperr tree)))
+
+(when #f
+  (let* (
+         (code "
+int foo() {
+  return bar( __extension__ (__attribute__((__vector_size__ (16))) int) 3);
+}")
+         (tree (parse-string
+                code
+                #:debug #f
+                )))
+    (pperr tree)))
+
+(when #f
+  ;;(pperr (get-sys-inc-dirs #:CC "clang"))
+  ;;(pperr (get-sys-inc-dirs #:CC "clang"))
+  (pperr (get-sys-inc-dirs))
+  )
+
+(when #f ;; sv bug 67664
+  (let* ((code "int foo(int *argv[]);")
+         (tree (parse-string code))
+         (udict (c99-trans-unit->udict tree))
+         (decl (assoc-ref udict "foo"))
+         (pdecl (sx-ref* decl 2 1 2 1))
+         (mdecl (udecl->mdecl pdecl))
+         )
+    ;;(pperr tree)
+    ;;(pperr pdecl)
+    ;;(pperr mdecl)
+    (pperr (ccode->sexp code))
+    ))
+
+;; issue#25
+(when #f
+  (let* ((code "
+#include <stdint.h>
+struct rseq_cs {
+	/* Version of this structure. */
+	uint32_t version;
+	/* enum rseq_cs_flags */
+	uint32_t flags;
+	uint64_t start_ip;
+	/* Offset from start_ip. */
+	uint64_t post_commit_offset;
+	uint64_t abort_ip;
+} __attribute__((aligned(4 * sizeof(uint64_t))));"))
+    (parse-string code)))
+
+;; issue#26
+(when #f
+  (let* ((code "#include <linux/elf.h>\n")
+         (tree (parse-string code #:mode 'decl))
+         (udict (c99-trans-unit->udict/deep tree))
+         )
+    ;;(pp (ccode->sexp code))
+    ;;(pp udict)
+    (if #f #f)))
+
+(when #f
+  (let* ((code "#include <asm-generic/siginfo.h>\n")
+         (tree (parse-string code #:mode 'decl))
+         (udict (c99-trans-unit->udict/deep tree))
+         )
+    ;;(pp (ccode->sexp code))
+    ;;(pp udict)
+    (if #f #f)))
+
+;; issue#36
+(when #f
+  (let* ((code "int f() {
+__asm__(\"bswapl %0 ; bswapl %1 ; xchgl %0,%1\"
+	    : \"=r\" (v.s.a), \"=r\" (v.s.b)
+	    : \"0\" (v.s.a), \"1\" (v.s.b));
+	
+}"))
+    (pp (parse-string code))))
+
+;; issue#49
+(when #f
+  (let ((code "int foo(int x) asm (\"\" \" foo64\");"))
+    ;;(pp (parse-string code))
+    (pp (ccode->sexp code))))
 
 ;; --- last line ---
