@@ -1,6 +1,6 @@
 ;;; nyacc/lang/c99/c99eval.scm - evaluate constant expressions
 
-;; Copyright (C) 2018-2024 Matthew Wette
+;; Copyright (C) 2018-2026 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,7 @@
             cx-incr-bit-size
             cx-maxi-size)
   #:use-module (nyacc lalr)
-  #:use-module (nyacc parse)
+  #:use-module (nyacc parse)            ; may need to convert to c99/parse
   #:use-module (nyacc lex)
   #:use-module (nyacc util)
   #:use-module ((nyacc lang util) #:select (make-tl tl-append tl->list))
@@ -45,7 +45,15 @@
   #:use-module (rnrs arithmetic bitwise)
   #:use-module (system foreign)
   #:use-module (nyacc foreign arch-info)
-  #:use-module (ice-9 match))
+  #:use-module (nyacc lang c99 cxmach))
+(cond-expand
+ (mes
+  (use-modules (smatch))
+  (define-macro (match exp . clauses)
+    `((smatch-lambda . ,clauses) ,exp)))
+ (else
+  (use-modules (ice-9 match))))
+
 
 (use-modules (ice-9 pretty-print))
 (define (sferr fmt . args)
@@ -57,8 +65,11 @@
 (define (sizeof-string-const value)
   #f)
 
-(include-from-path "nyacc/lang/c99/mach.d/c99cx-act.scm")
-(include-from-path "nyacc/lang/c99/mach.d/c99cx-tab.scm")
+(define c99cx-tables
+  (map (lambda (key) (cons key (assq-ref c99cx-mach key)))
+       '(mtab ntab len-v rto-v pat-v)))
+(define c99cx-act-v (assq-ref c99cx-mach 'act-v))
+(define c99cx-mtab (assq-ref c99cx-mach 'mtab))
 
 (define c99cx-raw-parser
   (make-lalr-parser
@@ -171,6 +182,8 @@
      (let ((mult (eval-c99-cx dim udict)))
        (call-with-values (lambda () (sizeof-mtail rest udict))
          (lambda (size align) (values (* mult size) align)))))
+    (`((array-of) . ,rest)              ; need to detect this later
+     (sizeof-mtail `((array-of "0") . ,rest) udict))
     (`((struct-def (field-list . ,fields)))
      (do-aggr fields incr-size))
     (`((struct-def (ident ,name) (field-list . ,fields)))
