@@ -1,6 +1,6 @@
 ;;; nyacc/lang/c99/munge-base.scm -
 
-;; Copyright (C) 2015-2018,2020,2023-2024 Matthew Wette
+;; Copyright (C) 2015-2018,2020,2023-2025 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -43,9 +43,16 @@
   #:use-module (nyacc lang sx-util)
   #:use-module (srfi srfi-11)           ; let-values
   #:use-module ((srfi srfi-1) #:select (fold fold-right))
-  #:use-module (ice-9 match)
   #:use-module (ice-9 hash-table)
   #:use-module (ice-9 vlist))
+(cond-expand
+ (mes
+  (use-modules (nyacc lang c99 cppmach))
+  (use-modules (smatch))
+  (define-macro (match exp . clauses)
+    `((smatch-lambda . ,clauses) ,exp)))
+ (else
+  (use-modules (ice-9 match))))
 
 (use-modules (ice-9 pretty-print))
 (define (sferr fmt . args)
@@ -69,7 +76,7 @@
     ((ftn-declr ,dir-declr . ,rest) (declr-ident dir-declr))
     ((scope ,declr) (declr-ident declr))
     ((bit-field ,ident ,size) ident)
-    ((bit-field ,size) `(ident ""))      ; ???
+    ((bit-field ,size) `(ident "*anon*"))
     (,_ (throw 'c99-error "c99/munge: unknown declarator: ~S" declr))))
 
 ;; @deffn {Procedure} declr-name declr => "name"
@@ -378,7 +385,8 @@
              (values specl declr)
              (values (replace-type-spec specl '(type-spec (void))) declr)))
         (else
-         (re-expand (replace-aggr-ref specl name (w/struct name) udict) declr))))
+         (re-expand
+          (replace-aggr-ref specl name (w/struct name) udict) declr))))
 
       ((union-ref (ident ,name))
        (cond
@@ -438,7 +446,7 @@
          (let ((tspec '(type-spec (fixed-type "int")))) ;; FIXME packed?
            (values (replace-type-spec specl tspec) declr)))))
 
-      (,_ (values specl declr)))))
+      (,_ (values specl declr))))) ;; <= typeof-{expr,type} will be skipped
 
 ;; @deffn {Procedure} expand-typerefs adecl udict [keep]
 ;; Given a declaration or component-declaration, return a udecl with all
@@ -638,11 +646,13 @@
 
   (define (unwrap-declr declr tail)
     (sx-match declr
+      ;; FIXME tossing initializers - e.g. double d[] = { 1.0, 2.0, 3.0 };
+      ;; FIXME if asm-expr is asm name we don't transfer that
       ((init-declr ,item) (unwrap-declr item tail))
-      ((init-declr ,item (initzer . ,vals))
-       ;;(sferr "udecl->mdecl tossing initializer\n")
-       ;; FIXME this is bad for: double d[] = { 1.0, 2.0, 3.0 };
-       (unwrap-declr item tail)) ;; ?
+      ((init-declr ,item (initzer . ,vals)) (unwrap-declr item tail))
+      ((init-declr ,item (asm-expr . ,vals)) (unwrap-declr item tail))
+      ((init-declr ,item (asm-expr . ,vals) (initzer . ,vals))
+       (unwrap-declr item tail))
       ((comp-declr ,item) (unwrap-declr item tail))
       ((param-declr ,item) (unwrap-declr item tail))
       ((param-declr) (cons (namer) tail))
@@ -660,14 +670,15 @@
        (unwrap-declr dcl (cons `(function-returning ,param-list) tail)))
       ((abs-ftn-declr ,param-list)
        (cons* (namer) `(function-returning ,param-list) tail))
+      ((ftn-declr ,dcl ,param-list ,asm-expr)
+       (unwrap-declr dcl (cons `(function-returning ,param-list) tail)))
       ((scope ,expr) (unwrap-declr expr tail))
       ((bit-field (ident ,name) ,size) (cons* name `(bit-field ,size) tail))
-      ((bit-field ,size) (cons* "" declr tail))
+      ((bit-field ,size) (cons* "*anon*" declr tail))
       (,_
        (sferr "munge-base/unwrap-declr missed:\n") (pperr declr)
        (throw 'c99-error "munge-base/unwrap-declr failed")
        #f)))
-
   (unwrap-declr declr tail))
 
 ;; @deffn {Procedure} udecl->mdecl udecl [#:namer def-namer]
