@@ -1,6 +1,6 @@
 ;;; nyacc/lang/c99/munge.scm - util's for processing output of the C99 parser
 
-;; Copyright (C) 2015-2022 Matthew R. Wette
+;; Copyright (C) 2015-2022,2025 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,8 @@
 ;; 5) dictize replaces old unitize, w/ unitize making more sense now
 ;;    a) unitize: (int (x y)) -> ((int x) (int y))
 ;;    b) dictize: (int (x y)) -> (("x" . (int x)) ("y" . (int y)))
-;; 6) In places where a name does not exist (e.g., bitfield) "" is the key.
+;; 6) In places where a name does not exist (e.g., bitfield)
+;;    "*anon*" is the key.
 
 ;; Todo:
 ;; 2) I want a way to keep named enums in expand-typerefs.
@@ -82,7 +83,6 @@
                udecl->mdecl split-udecl
                declr-ident declr-name
                clean-field-list clean-fields)
-  #:use-module (ice-9 match)
   #:use-module ((srfi srfi-1) #:select (fold fold-right remove))
   #:use-module (srfi srfi-11)           ; let-values
   #:use-module ((sxml fold) #:select (foldts foldts*))
@@ -96,7 +96,14 @@
   ;; debugging:
   #:use-module (system vm trace)
   #:use-module (ice-9 pretty-print))
-;; undocumented Guile builtins: or-map
+(cond-expand
+ (mes
+  (use-modules (nyacc lang c99 cppmach))
+  (use-modules (smatch))
+  (define-macro (match exp . clauses)
+    `((smatch-lambda . ,clauses) ,exp)))
+ (else
+  (use-modules (ice-9 match))))
 
 (define (sferr fmt . args) (apply simple-format (current-error-port) fmt args))
 (define (pperr exp)
@@ -106,14 +113,15 @@
 (define tmap-fmt
   '(("char" "%hhd")
     ("unsigned char" "%hhu")
-    ("short int" "%hd")
-    ("unsigned short int" "%hu")
+    ("signed char" "%hhd")
+    ("short" "%hd")
+    ("unsigned short" "%hu")
     ("int" "%d")
-    ("unsigned int" "%u")
-    ("long int" "%ld")
-    ("unsigned long int" "%lu")
-    ("long long int" "%lld")
-    ("unsigned long long int" "%llu")))
+    ("unsigned" "%u")
+    ("long" "%ld")
+    ("unsigned long" "%lu")
+    ("long long" "%lld")
+    ("unsigned long long" "%llu")))
 
 ;; @deffn {Procedure} attr-append . attr-lists
 ;; This is hack for now: it is @code{append}. It needs to be set up to combine
@@ -135,13 +143,13 @@
 ;; @end example
 
 ;; mdecl is
-;; ("foo" (pointer-to) (ary-declr 3) (fixed-type "unsigned int"))
+;; ("foo" (pointer-to) (ary-declr 3) (fixed-type "unsigned"))
 ;; which can be converted to
-;; ("(*foo) (ary-declr 3) (fixed-type "unsigned int"))
+;; ("(*foo) (ary-declr 3) (fixed-type "unsigned"))
 ;; which can be converted to
-;; (("((*foo)[0])" (fixed-type "unsigned int"))
-;;  ("((*foo)[1])" (fixed-type "unsigned int"))
-;;  ("((*foo)[2])" (fixed-type "unsigned int"))
+;; (("((*foo)[0])" (fixed-type "unsigned"))
+;;  ("((*foo)[1])" (fixed-type "unsigned"))
+;;  ("((*foo)[2])" (fixed-type "unsigned"))
 
 ;; may need to replace (typename "int32_t") with (fixed-type "int32_t")
 
@@ -171,7 +179,7 @@
        (lambda (declr seed)
          (acons (declr-name declr) (sx-list tag attr specl declr) seed))
        seed declrs)
-      (acons "" (sx-list tag attr specl) seed)))
+      (acons "*anon*" (sx-list tag attr specl) seed)))
 
 ;; @deffn {Procedure} split-decl decl => values
 ;; This routine splits a declaration (or comp-decl or param-decl) into
@@ -435,7 +443,7 @@
 ;; solution: unitize-decl => '(struct . "foo") then filter to generate
 ;; ("struct" ("foo" . decl) ("bar" . decl) ...)
 ;; ("union" ("bar" . decl) ("bar" . decl) ...)
-;; ("enum" ("" . decl) ("foo" . decl) ("bar" . decl) ...)
+;; ("enum" ("*anon*" . decl) ("foo" . decl) ("bar" . decl) ...)
 ;; @end example
 ;; So globals could be in udict, udefs or anon-enum.
 ;; @example
@@ -598,20 +606,24 @@
 (define* (udict-enums->ddict udict #:optional (ddict '()))
   (define (gen-nvl edef-list ddict)
     (let ((def-list (and=> (canize-enum-def-list edef-list udict ddict) cdr)))
-      (fold (lambda (edef dd) (acons (sx-ref* edef 1 1) (sx-ref* edef 2 1) dd))
+      (fold (lambda (edef dd)
+              (acons (sx-ref* edef 1 1) (sx-ref* edef 2 1) dd))
             ddict def-list)))
   (fold
    (lambda (pair ddict)
+     ;; not sure why I'm filtering out ("" . decl)
+     ;;     => because it's a duplicate to "*anon*"
+     ;;        ("" . decl) should be deprecated
      (if (or (pair? (car pair)) (positive? (string-length (car pair))))
          (let* ((specs (sx-ref (cdr pair) 1))
-                (tspec (sx-ref (sx-find 'type-spec specs) 1)))
-           (sx-match tspec
+                (tspec (sx-find 'type-spec specs))
+                (tspec (and tspec (sx-ref tspec 1))))
+           (sx-match tspec ;; tspec may be #f if typeof-{expr,type}
              ((enum-def (ident ,name) (enum-def-list . ,defs))
               (gen-nvl (sx-ref tspec 2) ddict))
              ((enum-def (enum-def-list . ,defs))
               (gen-nvl (sx-ref tspec 1) ddict))
-             (,_
-              ddict)))
+             (,_ ddict))) ;; <= can be typeof
          ddict))
    ddict udict))
 
@@ -851,7 +863,7 @@
       (`((struct-def . ,rest)) (make-udecl (car mdecl-tail) declr))
       (`((union-def . ,rest)) (make-udecl (car mdecl-tail) declr))
       (`((enum-def . ,rest))  (make-udecl (car mdecl-tail) declr))
-      (__ (sferr "munge/mdecl->udecl missed:\n")
+      (_ (sferr "munge/mdecl->udecl missed:\n")
           (pperr mdecl-tail)
           (throw 'nyacc-error "munge/mdecl->udecl failed"))))
 
