@@ -1,6 +1,6 @@
 ;;; module/nyacc/sx-util.scm - runtime utilities for the parsers
 
-;; Copyright (C) 2015-2018 Matthew R. Wette
+;; Copyright (C) 2015-2018,2025 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,7 @@
             sx-has-attr? sx-attr-ref sx-attr-add sx-attr-add* sx-attr-set!
             sx-find
             sx-split sx-split* sx-join sx-join* sx-cons sx-cons* sx-list
+            sx-cons*/src sx-list/src
             sx-unitize
             sx-match sx-match-tail
             ;; deprecated:
@@ -285,6 +286,18 @@
 (define sx-join sx-cons)
 (define sx-join* sx-list)
 
+;; @deffn {Procedure} sx-cons*/src src tag attr exp ... tail => sexp
+;; @deffnx {Procedure} sx-list/src src tag attr exp ... => sexp
+;; These are like their respective procedures without the @code{/src}
+;; but allow source properties to be transferred from the sx @var{src}.
+;; @end deffn
+(define (sx-cons*/src src tag attr . rest)
+  (let ((sx (apply sx-cons* tag attr rest)))
+    (cons-source src (car sx) (cdr sx))))
+(define (sx-list/src src tag attr . rest)
+  (let ((sx (apply sx-list tag attr rest)))
+    (cons-source src (car sx) (cdr sx))))
+
 ;; @deffn {Procedure} sx-split sexp => tag attr tail
 ;; @deffnx {Procedure} sx-split* sexp => tag attr exp ...
 ;; Split an SXML element into its constituent parts, as a @code{values},
@@ -326,6 +339,8 @@
 ;; broken up in order to not overflow the stack during compilation.
 ;; This uses only syntax-rules; sxml uses syntax-case.
 
+;; update ,_ is "don't care" (so no binding generated)
+
 
 ;; sx-haz-attr? val
 ;;(define (sx-haz-attr? sx)
@@ -367,9 +382,9 @@
 ;; kt kf are continuation syntax expresions
 
 ;; @deffn {Syntax} sx-match exp (pat body ...) ...
-;; This syntax will attempt to match @var{expr} against the patterns.
+;; This syntax will attempt to match the SXML @var{exp} against the patterns.
 ;; At runtime, when @var{pat} is matched against @var{exp}, then @var{body ...}
-;; will be evaluated.
+;; will be evaluated.  The pattern @code{()} is also supported.
 ;; @end deffn
 (define-syntax sx-match
   (syntax-rules ()
@@ -396,30 +411,39 @@
     ((_ v) (error "sx-match-tail: nothing matches"))))
 
 ;; sxm-sexp val pat kt kf
-;; match sexp
+;; match sexp @var{val}
 (define-syntax sxm-sexp
-  (syntax-rules (@ unquote)
+  (syntax-rules (@ unquote _)
     ;; accept anything
+    ((_ v (unquote _) kt kf) kt)
     ((_ v (unquote w) kt kf) (let ((w v)) kt))
     ;; capture attributes by name
     ((_ v (tag (@ (ky vl) p1 ...) . nl) kt kf)
-     (sxm-tag (car v) tag
-              (let ((va (sx-attr v)) (vt (sx-tail v)))
-                (sxm-attr-tail va vt ((ky vl) p1 ...) nl kt kf))
-              kf))
+     (if (pair? v)
+         (sxm-tag (car v) tag
+                  (let ((va (sx-attr v)) (vt (sx-tail v)))
+                    (sxm-attr-tail va vt ((ky vl) p1 ...) nl kt kf))
+                  kf)
+         kf))
     ;; capture attributes as dict
     ((_ v (tag (@ . (unquote va)) . nl) kt kf)
-     (sxm-tag (car v) tag
-              (let ((va (sx-attr v)) (vt (sx-tail v)))
-                (sxm-tail vt nl kt kf))
-              kf))
+     (if (pair? v)
+         (sxm-tag (car v) tag
+                  (let ((va (sx-attr v)) (vt (sx-tail v)))
+                    (sxm-tail vt nl kt kf))
+                  kf)
+         kf))
     ;; ignore attributes; (cadr v) may be an attr node. If so, ignore it.
     ((_ v (tag . nl) kt kf)
-     (sxm-tag (car v) tag
-              (let ((vt (sx-tail v)))
-                (sxm-tail vt nl kt kf))
-              kf))
-    ))
+     (if (pair? v)
+         (sxm-tag (car v) tag
+                  (let ((vt (sx-tail v)))
+                    (sxm-tail vt nl kt kf))
+                  kf)
+         kf))
+    ;; allow end-of-list
+    ((_ v () kt kf)
+     (if (null? v) kt kf))))
  
 ;; sxml-attr-tail va vt (@ (k v) ...) nl kt kf
 (define-syntax sxm-attr-tail
@@ -462,8 +486,9 @@
 ;; [ht][vp] = [head,tail][value,pattern]
 ;; Can this be set up to match a string constant?
 (define-syntax sxm-node
-  (syntax-rules (unquote)
+  (syntax-rules (unquote _)
     ((_ v () kt kf) (if (null? v) kt kf))
+    ((_ v (unquote _) kt kf) kt)
     ((_ v (unquote w) kt kf) (let ((w v)) kt))
     ((_ v (hp . tp) kt kf) (if (pair? v) (sxm-sexp v (hp . tp) kt kf) kf))
     ((_ v s kt kf)
