@@ -1,6 +1,6 @@
 ;;; scripts/compile-ffi.scm --- NYACC's command-line FFI compiler
 
-;; Copyright (C) 2017-2021,2024-2025 Matthew Wette
+;; Copyright (C) 2017-2021,2024-2026 Matthew Wette
 ;;
 ;; This program is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU Lesser General Public License as published by
@@ -24,14 +24,16 @@
 ;;; Code:
 
 (define-module (scripts compile-ffi)
-  #:use-module (nyacc foreign arch-info)
   #:use-module (system base language)
   #:use-module ((system base compile) #:select (compile-file))
   #:use-module ((srfi srfi-1) #:select (fold fold-right lset-union every))
   #:use-module (srfi srfi-37)
-  #:version (2 02 5))
+  #:use-module (nyacc foreign arch-info)
+  #:use-module ((nyacc lang c99 ffi-help)
+                #:select (compile-ffi-file *fh-backend*))
+  #:version (3 03 0))
 
-(define *ffi-help-version* "2.02.5")
+(define *ffi-help-version* "3.03.0")
 
 (define (compile-scm file)
   (compile-file file
@@ -80,7 +82,7 @@ Generate a Guile Scheme file from the source FFI file FILE.
   -h, --help            print this help message
   --version             print version number
 
-  -b, --backend=BACKEND back end target: bytestructures or cdata
+  -b, --backend=BACKEND backend (default: cdata)
   -L, --load-path=DIR   add DIR to the front of the module load path
   -I, --inc-dir=DIR     add DIR to list of dir's to search for C headers
   -o, --output=OFILE    write output to OFILE
@@ -92,9 +94,11 @@ Generate a Guile Scheme file from the source FFI file FILE.
   -X, --no-exec         don't generate .go file(s)
   -R, --dont-recurse    don't do recursive compile on dep's
   -a, --any-suffix      allow any suffix (if not, must be .ffi)
+  -w, --no-foreign-library  guile has no (system foreign-library) module
+  -x, --external-cdata  for cdata backend, use external cdata package
 
 See the FFI-Helper User's Manual for help generating a .ffi file.
-Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
+Report bugs to https://github.com/mwette/nyacc/issues.\n"))
 
 (define options
   ;; specification of command-line options
@@ -121,7 +125,7 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
              (values (acons 'output-dir arg opts) files)))
    (option '(#\b "backend") #t #f
            (lambda (opt name arg opts files)
-             (values (acons 'backend (string->symbol arg) opts) files)))
+             (values (acons 'backend arg opts) files)))
    (option '(#\m "machine") #t #f
            (lambda (opt name arg opts files)
              (values (acons 'machine arg opts) files)))
@@ -145,13 +149,20 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
              (values (acons 'no-exec #t opts) files)))
    (option '(#\a "any-suffix") #f #f
            (lambda (opt name arg opts files)
-             (values (acons 'any-suffix #t opts) files)))))
+             (values (acons 'any-suffix #t opts) files)))
+   (option '(#\w "no-foreign-library") #f #f
+           (lambda (opt name arg opts files)
+             (values (acons 'no-foreign-library #t opts) files)))
+   (option '("external-cdata") #f #f
+           (lambda (opt name arg opts files)
+             (values (acons 'ext-cdata #t opts) files)))
+   ))
 
 ;; from scripts/compile.scm
 (define (parse-args args)
   (args-fold args
              options
-             (lambda (opt name arg files opts)
+             (lambda (opt name arg opts files)
                (fail "unrecognized option: ~S" name)
                (exit 1))
              (lambda (file opts files)
@@ -159,9 +170,14 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
                    (string-suffix? ".ffi" file)
                    (fail "expecting .ffi suffix"))
                (values opts (cons file files)))
-             `((backend . ,(or (getenv "FFI_HELP_BACKEND") 'cdata))
+             `((backend . ,(getenv "FFI_HELP_BACKEND"))
                (machine . "native")) '()))
 
+(define (basename/tag path)
+  (let* ((base (basename path))
+         (tagix (string-rindex base #\.)))
+    (substring base 0 tagix)))
+
 ;; --- check dependencies -------------------------------------------
 
 (define-syntax find-ffi-uses
@@ -252,8 +268,9 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
   (cond
    ((assq-ref opts 'output))
    ((assq-ref opts 'output-dir) =>
-    (lambda (dir) (string-append dir "/" (basename ffi-file ".ffi") ".scm")))
-   (else (string-append (string-drop-right ffi-file 4) ".scm"))))
+    (lambda (dir) (string-append dir "/" (basename/tag ffi-file) ".scm")))
+   (else
+    (string-append (substring ffi-file 0 (string-rindex ffi-file #\.)) ".scm"))))
 
 ;; from list of all files and depd, return supplier-for dict
 (define (ensure-ffi-deps file opts)
@@ -295,16 +312,19 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
 
 (define (compile-ffi ffi-file opts)
   (let* ((scm-file (scm-for-ffi ffi-file opts))
-         (mod (case (assq-ref opts 'backend)
-                ((bytestructures bs) '(nyacc lang c99 ffi-help-bs))
-                ((cdata cd) '(nyacc lang c99 ffi-help-cd))
-                (else (fail "bad backend: ~s" (assq-ref opts 'backend)))))
-         (compile-ffi-file (module-ref (resolve-module mod) 'compile-ffi-file)))
+         (fhbe (assq-ref opts 'backend))
+         (fhbe-module (and fhbe (map string->symbol (string-split fhbe #\/))))
+         (bemod (and fhbe (resolve-module fhbe-module)))
+         (backend (and bemod (false-if-exception (module-ref bemod 'backend)))))
     (when (assq-ref opts 'list-deps)
       (for-each
        (lambda (f) (sfmt "~A\n" (fix-path f)))
        (apply lset-union string=? (all-ffi-deps ffi-file)))
       (exit 0))
+    (if fhbe
+        (if (not backend)
+            (fail "backend not resolved: ~s" fhbe)
+            (*fh-backend* backend)))
     (catch 'ffi-help-error
       (lambda ()
         (catch 'c99-error
@@ -330,8 +350,8 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
     (lambda (opts files)
       (when (or (assq-ref opts 'help) (null? files)) (show-usage) (exit 0))
       (unless (string=? (assq-ref opts 'machine) "native")
-        (unless (memq (assq-ref opts 'backend) '(cdata))
-          (fail "only cdata supports non-native machines architectures"))
+        (when (assq-ref opts 'backend)
+          (fail "only default cdata supports non-native machines architectures"))
         (unless (getenv "CC")
           (fail "for non-native machine architectures define env var CC")))
       (for-each
